/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/

Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");
Components.utils.import("resource://calendar/modules/calUtils.jsm");

(function() {
  // Thunderbird
  if (!Components.interfaces.calICalendar) {
    return;
  }

  // Init logger
  let lLogger = Components.classes["@obm.org/calendar/logger;1"].createInstance(Components.interfaces.obmILogger);
  let lwriter = Components.classes["@obm.org/calendar/writer;1?type=unixfile"].getService(Components.interfaces.obmILoggerWriter);
  lLogger.setLevelWriter(gLogger.LOAD_LEVEL, lwriter);
  lLogger.setHeader("=== lightningOverlay: ");

  lLogger.debug("subscribeToOBMCalendar: Setting pref calendar.itip.notify=false", false, false);
  UtilsOBM.setPrefSafe("calendar.itip.notify", false);

  // autoconf : subscribe to calendar and remove lightning default calendar
  let autoconfedLogin = UtilsOBM.getPrefSafe("extensions.obm.autoconf.login", null);
  let autoconfedEmail = UtilsOBM.getPrefSafe("extensions.obm.autoconf.email", null);
  let autoconfRemoveDefaultCalendar = UtilsOBM.getPrefSafe("extensions.obm.autoconf.removedefaultcalendar", false);
  if (autoconfedLogin) {
    if (!UtilsOBM.getOBMCredentials({}, {})) {
      UtilsOBM.setOBMCredentials(autoconfedLogin, "");
      lLogger.debug("login intialized with autconfed login " + autconfedlogin, false, false);
    }

    let obmCalMgr = UtilsOBM.getCalendarManager();

    if (autoconfedEmail) {
      if (obmCalMgr.getOBMCalendars({}).length == 0) {
        // subscribe to the calendar
        let name = UtilsOBM.getPrefSafe("extensions.obm.autoconf.calname", null) || autoconfedLogin;
        let calendar = obmCalMgr.subscribeObmCalendar(autoconfedLogin.split("@")[0],
                                                      name, autoconfedEmail, false);

        // select in composite
        cal.getCalendarManager().setCalendarPref_(calendar, "calendar-main-in-composite", true);
        let compositeCalendar = Components.classes["@mozilla.org/calendar/calendar;1?type=composite"]
                                          .createInstance(Components.interfaces.calICompositeCalendar);
        compositeCalendar.prefPrefix = "lightning-main";
        compositeCalendar.addCalendar(calendar);
        compositeCalendar.defaultCalendar = calendar;
        lLogger.debug("calendar subscribed with autoconfed email " + autoconfedEmail, false, false);
      }
    }

  }
  if (autoconfRemoveDefaultCalendar) {
    // Delete at startup if this is the local calendar
    var calendarManager = cal.getCalendarManager();
    var calendars = calendarManager.getCalendars({});
    for each(var calendar in calendars) {
      if (calendar.uri.spec == "moz-profile-calendar://" || calendar.uri.spec == "moz-storage-calendar://") {
        calendarManager.unregisterCalendar(calendar);
        calendarManager.deleteCalendar(calendar);
        lLogger.debug("default calendar removed by autoconf", false, false);
      }
    }
  }
})();

function OBM_InitUI() {
    let info = Components.classes["@mozilla.org/xre/app-info;1"]
                         .getService(Components.interfaces.nsIXULAppInfo);

    let vc = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
                       .getService(Components.interfaces.nsIVersionComparator);

    if (vc.compare(info.version, "11.0") >= 0) {
        OBM_InitUI_TB11();
    } else {
        OBM_InitUI_TB10();
    }
}

function OBM_getLastSpring(bar) {
    let lastSpring = null;
    let springs = bar.getElementsByTagName("toolbarspring");
    if (springs && springs.length) {
        lastSpring = springs[springs.length - 1];
    }
    return lastSpring;
}

// Until Thunderbird 10, there was just one toolbar. This also covers
// Thunderbird 3. Here we will do standard synchronization using the dialog, a
// synchronize button is also inserted.
function OBM_InitUI_TB10() {
    let buttonIds = ["obm-sync-button", "obm-web-button", "obm-account-button"];
    let mailbar = document.getElementById("mail-bar3");
    let mailspring = OBM_getLastSpring(mailbar);
    for each (let button in buttonIds) {
        mailbar.insertItem("mail-" + button, mailspring);
    }
}

// The new toolbar was introduced in Lightning 1.3 = Thunderbird 11. Here we
// will do synchronization in the background.
function OBM_InitUI_TB11() {
    let buttonIds = ["obm-web-button", "obm-account-button"];
    let calbar = document.getElementById("calendar-toolbar2");
    let taskbar = document.getElementById("task-toolbar2");
    let calspring = OBM_getLastSpring(calbar);
    let taskspring = OBM_getLastSpring(taskbar);

    for each (let button in buttonIds) {
        calbar.insertItem("calendar-" + button, calspring);
        taskbar.insertItem("task-" + button, taskspring);
    }

    // Hook up our sync dialog to Lightning's reload remote calendars command
    (function injectController() {
        let controller = {
            defaultControler: null,
            isCommandEnabled: function(aCommand) {
                return this.defaultController.isCommandEnabled(aCommand);
            },
            supportsCommand: function(aCommand) {
                return aCommand == "calendar_reload_remote_calendars";
            },
            doCommand: function(aCommand) {
                if (aCommand == "calendar_reload_remote_calendars") {
                    setTimeout(function() {
                        let syncManager = UtilsOBM.getSyncManager();
                        syncManager.syncWithDialog();
                    }, 0);
                }
                this.defaultController.doCommand(aCommand);
            }
        };
        // Make sure our controller is inserted before the lightning controller
        let ltnController = top.controllers.getControllerForCommand("calendar_reload_remote_calendars");
        if (!ltnController) {
            setTimeout(injectController, 0);
        } else {
            controller.defaultController = ltnController;
            top.controllers.insertControllerAt(0, controller);
        }
    })();
}
window.addEventListener("load", OBM_InitUI, false);
