/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/

Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");
Components.utils.import("resource://calendar/modules/calUtils.jsm");

var DEFAULT_AUTOSYNC_DELAY = 5000;
var DEFAULT_DELAY_FOR_DND_TITLE_EDIT = 30000;
var DEFAULT_SYNC_EVENTS_WITH_NO_ATTENDEES = true;

/* Calendar Event Synchronizer */

if ( typeof(Calendar_Base_Synchronizer) == "undefined" ) {
    let scriptLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
                                 .createInstance(Components.interfaces.mozIJSSubScriptLoader);
    scriptLoader.loadSubScript("chrome://obm-extension/content/synchronizer/Calendar_Base_Synchronizer.js");
}

function Calendar_Event_Synchronizer() {
    this.mSupportedItemType = Components.interfaces.calIEvent;
    this.mSyncManager = UtilsOBM.getSyncManager();
    this.mWindowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
    this.mQueuedNotifications = [];
    this.mSynchronizingCalendars = false;
    
    Services.obs.addObserver(this, "obmCalendarSyncCompleted", false);
    Services.obs.addObserver(this, "obmCalendarSyncStarted", false);
}

Calendar_Event_Synchronizer.prototype = {
  __proto__: new Calendar_Base_Synchronizer(),
  

  observe: function(aSubject, aTopic, aData) {
      switch (aTopic) {
          case "obmCalendarSyncStarted":
              this.mSynchronizingCalendars = true;
              break;
          case "obmCalendarSyncCompleted":
              this.mSynchronizingCalendars = false;
              this.onCalendarSyncCompleted();
      }
  },
  
  onCalendarSyncCompleted: function () {
      // OBMFULL-4473
      // Now that the sync is done, we should process all queued notifications
      if (this.mQueuedNotifications) {
          while (this.mQueuedNotifications.length > 0) {
              let notification = this.mQueuedNotifications.shift();
              
              this.logger.debug("Event_Synchronizer: Executing queued notification: " + notification.func);
              
              notification.func.apply(this, notification.args);
          }
      }
  },
  
  notify: function (func, aContainer, aItem, type, generateIdIfMissing) {
      if (this.mSynchronizingCalendars) {
          this.mQueuedNotifications.push({
              func: func,
              args: [aContainer, aItem]
          });
          
          this.logger.notice("Event_Synchronizer: Queued '" + type + "' notification as we're synchronizing.");
          
          return;
      }
      
      this.logger.debug("Event_Synchronizer: " + type, false, false);

      aContainer = aContainer.QueryInterface(this.mSupportedContainerType);
      aItem = aItem.QueryInterface(this.mSupportedItemType);

      if (!this.isContainerManagedByOBM(aContainer)) {
          return;
      }

      let itemID = this.getItemID(aItem);
      
      if (!itemID) {
          if (!generateIdIfMissing) {
              return;
          }
          
          itemID = "" + Math.round(Math.random() * 1000) + "-" + Math.round(Math.random() * 1000);
          
          this.logger.debug("Event_Synchronizer: Generated item ID: " + itemID, false, false);
      }

      UtilsOBM.addEventToChangesList(aItem, type);

      if (generateIdIfMissing) {
          this.setItemID(aContainer, aItem, itemID);
      }

      this.syncCalendar(aContainer, aItem);
  },
  
  notifyItemAdded: function(aContainer, aItem) {
      this.notify(this.notifyItemAdded, aContainer, aItem, "added", true);
  },

  notifyItemModified: function(aContainer, aItem) {
      this.notify(this.notifyItemModified, aContainer, aItem, "updated");
  },

  notifyItemParticipationChanged: function(aContainer, aItem) {
      this.notify(this.notifyItemParticipationChanged, aContainer, aItem, "participationChanged");
  },

  notifyItemDeleted: function(aContainer, aItem) {
      this.notify(this.notifyItemDeleted, aContainer, aItem, "removed");
  },
  
  syncCalendar: function (aCalendar, aItem) {
      let proactiveSyncEnabled = UtilsOBM.getPrefSafe("extensions.obm.proactiveSync", false);
      let syncEventsWithNoAttendees = UtilsOBM.getPrefSafe("extensions.obm.proactiveSync.syncEventsWithNoAttendees", DEFAULT_SYNC_EVENTS_WITH_NO_ATTENDEES);

      if (!proactiveSyncEnabled || (!syncEventsWithNoAttendees && aItem.getAttendees({}).length == 0)) {
          return;
      }

      let delay = this.isDndEventCreation(aItem) ?
                      UtilsOBM.getPrefSafe("extensions.obm.proactiveSync.delayForDnDTitleEdit", DEFAULT_DELAY_FOR_DND_TITLE_EDIT)
                      : UtilsOBM.getPrefSafe("extensions.obm.proactiveSync.delay", DEFAULT_AUTOSYNC_DELAY);

      this.timer = this.timer || Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
      this.timer.cancel();
      this.timer.initWithCallback({
          synchronizer: this,
          notify: function(timer) {
              let synchronizer = this.synchronizer;
              let eventDialog = synchronizer.mWindowMediator.getMostRecentWindow("Calendar:EventDialog");
              let logger = synchronizer.logger;
              
              if (eventDialog) {
                  logger.notice("An event dialog is open, skipping background synchronization.", false, false);
                  
                  eventDialog.rescheduledCalendarSync = eventDialog.rescheduledCalendarSync || [];
                  if (eventDialog.rescheduledCalendarSync.indexOf(aCalendar) < 0) {
                      let originalDispose = eventDialog.dispose;
                      
                      eventDialog.dispose = function () {
                          originalDispose();
                          
                          logger.debug("Event dialog was closed, rescheduling background synchronization.", false, false);
                          synchronizer.syncCalendar(aCalendar, aItem);
                      };
                      eventDialog.rescheduledCalendarSync.push(aCalendar);
                  }
              } else {
                  synchronizer.mSyncManager.syncCalendar(aCalendar);
              }
          }
      }, delay, this.timer.TYPE_ONE_SHOT);
  },
  
  isDndEventCreation: UtilsOBM.isDefaultEventTitle

};
