/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");

const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

function ObmLogger() {
	var writer = null;
	this._writers = new Array(writer, writer, writer, writer);
	this._filters = new Array(null, null, null, null);
	this._header = "";
};


// static fields
const LEVEL_PREF = "extensions.obm.log.level";

//IDL: const long DEBUG_LEVEL = 0
ObmLogger.DEBUG_LEVEL = 0;
//IDL: const long NOTICE_LEVEL = 1
ObmLogger.NOTICE_LEVEL = 1;
//IDL: const long WARNING_LEVEL = 2
ObmLogger.WARNING_LEVEL = 2;
//IDL: const long ERROR_LEVEL = 3
ObmLogger.ERROR_LEVEL = 3;
//IDL: const long ALL_LEVELS = -1
//Use this constant for setting all levels at once
ObmLogger.ALL_LEVELS = -1; //
//IDL: const long LOAD_LEVEL = -2
//Use this constant to load level from pref PREF_LEVEL
ObmLogger.LOAD_LEVEL = -2; //

ObmLogger.prototype.classID = Components.ID("{8778a141-6b96-43ae-914b-3027b69ef4d3}");
ObmLogger.prototype.contractID = "@obm.org/calendar/logger;1";
ObmLogger.prototype.classDescription = "OBM Logger";
ObmLogger.prototype.QueryInterface = XPCOMUtils.generateQI([Ci.obmILogger]);


ObmLogger.prototype.setHeader = function (str) {
	this._header = str;
};


// instance methods
//IDL: ILogWriter getLevelWriter(in long level)
//Get writer for given level. Can be null if no writer.
ObmLogger.prototype.getLevelWriter = function (level) {
	if (level < 0 || level > ObmLogger.ERROR_LEVEL) {
		throw "level < 0 || level > ObmLogger.ERROR_LEVEL";
	}

	return this._writers[level];
};

//IDL: void setLevelWriter(in long level, in ILogWriter writer)
//Set writer for given level. Use writer=null for no writer.
ObmLogger.prototype.setLevelWriter = function (level, writer) {
	if (level != ObmLogger.LOAD_LEVEL && level > ObmLogger.ERROR_LEVEL) {
		throw "level != ObmLogger.LOAD_LEVEL && level > ObmLogger.LOAD_LEVEL";
	}

	if (level == ObmLogger.LOAD_LEVEL) {
        level = UtilsOBM.getPrefSafe(LEVEL_PREF, ObmLogger.ALL_LEVELS);
        if (level < -1 || level > 3) {
            level = -1;
        }
	}
	//dump("log level is :" + level + "\n");

	if (level != ObmLogger.ALL_LEVELS) {
		this._writers[level] = writer;
	} else {
		this._writers[ObmLogger.DEBUG_LEVEL] = writer;
		this._writers[ObmLogger.NOTICE_LEVEL] = writer;
		this._writers[ObmLogger.WARNING_LEVEL] = writer;
		this._writers[ObmLogger.ERROR_LEVEL] = writer;
	}
};

//IDL: string getLevelFilter(in long level)
ObmLogger.prototype.getLevelFilter = function (level) {
	if (level < 0 || level > ObmLogger.ERROR_LEVEL) {
		throw "level < 0 || level > ObmLogger.ERROR_LEVEL";
	}

	return this._filters[level];
};


//IDL: void setLevelFilter(in long level, in string filter)
ObmLogger.prototype.setLevelFilter = function (level, filter) {
	if (level != ObmLogger.ALL_LEVELS && level > ObmLogger.ERROR_LEVEL) {
		throw "level != ObmLogger.ALL_LEVELS && level > ObmLogger.ERROR_LEVEL";
	}

	if (level != ObmLogger.ALL_LEVELS) {
		this._filters[level] = filter;
	} else {
		this._filters[ObmLogger.DEBUG_LEVEL] = filter;
		this._filters[ObmLogger.NOTICE_LEVEL] = filter;
		this._filters[ObmLogger.WARNING_LEVEL] = filter;
		this._filters[ObmLogger.ERROR_LEVEL] = filter;
	}
};


//IDL: void debug(in string msg)
ObmLogger.prototype.debug = function (msg, showStack, showDate) {
	this._log(ObmLogger.DEBUG_LEVEL, msg, showStack, showDate);
};

//IDL: void notice(in string msg)
ObmLogger.prototype.notice = function (msg, showStack, showDate) {
	this._log(ObmLogger.NOTICE_LEVEL, msg, showStack, showDate);
};

//IDL: void warning(in string msg)
ObmLogger.prototype.warning = function (msg, showStack, showDate) {
	this._log(ObmLogger.WARNING_LEVEL, msg, showStack, showDate);
};

//IDL: void error(in string msg)
ObmLogger.prototype.error = function (msg, showStack, showDate) {
	this._log(ObmLogger.ERROR_LEVEL, msg, showStack, showDate);
};

ObmLogger.prototype._log = function (level, msg, showStack, showDate) {
	if (null === msg || 'string' !== typeof(msg)) return;
	if (null !== this._writers[level]) {
		if (null === this._filters[level] || -1 != msg.indexOf(this._filters[level])) {
			this._writers[level].write(this._formatLog(msg, level, showStack, showDate), level);
		}
	}
};

ObmLogger.prototype._formatLog = function (msg, level, showStack, showDate) {
	var str = "[" + this._levelNum2Text(level) + "] " + this._header;
	var dateString = "";
	var callStack = "";
	showDate=true;
	if (showDate) {
		dateString = new Date();
	}
	if (showDate) {
		str += "(" + dateString + ") ";
	}
	if (showStack) {
		var callStack = (level >= ObmLogger.DEBUG_LEVEL) ? this._formatCallStack(null) : '';
		if ('undefined' !== typeof(msg.stack)) {
			callStack = '(EXCEPTION) ' + this._formatCallStack(msg.stack);
		}
	}
	str += " > " + msg + "\n";
	if (showStack) {
		str += "Stack:\n" + callStack + "\n";
	}
	return str;
};

ObmLogger.prototype._levelNum2Text = function (level) {
	var level_txt = null;
	switch (level)
	{
		case ObmLogger.DEBUG_LEVEL:
		level_txt = 'DEBUG  ';
		break;
		case ObmLogger.NOTICE_LEVEL:
		level_txt = 'NOTICE ';
		break;
		case ObmLogger.WARNING_LEVEL:
		level_txt = 'WARNING';
		break;
		case ObmLogger.ERROR_LEVEL:
		level_txt = 'ERROR  ';
		break;
		case ObmLogger.ALL_LEVELS:
		level_txt = 'ALL LEVELS';
		break;
		default :
		level_txt = 'UNKNOWN LEVEL [' + level + '] Vs [' + this.DEBUG_LEVEL + ']';
		break;
	}
	return level_txt;
};

ObmLogger.prototype._formatCallStack = function (s) {
	var stack = (null === s) ? Components.stack : s;
	var buf = '';
	var depth = 0;
	while (null !== stack) {
		for (var i = 0; i < depth; i++) buf += '>';
		buf += ' ';
		if (depth > 3) buf += stack.toString() + '\n';
		depth++;
		stack = stack.caller;
	}
	return buf;
};

// module registration
var components = [ObmLogger];
if ("generateNSGetFactory" in XPCOMUtils) {
    var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
} else {
    var NSGetModule = XPCOMUtils.generateNSGetModule(components);
}
