/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

const Cc = Components.classes;
const Ci = Components.interfaces;

/**
 * component to access to the addressbook database
 */
function ObmAddressBook(aUid) {
    this.mUid = aUid;
    this.abURI = "moz-abmdbdirectory://alia-obm-" + aUid + ".mab";
    this.mDirectory = null; // nsIAbDirectory

    let addressBookManager = Components.classes["@mozilla.org/abmanager;1"]
                                    .getService(Components.interfaces.nsIAbManager);
    let dirPrefId = "ldap_2.servers." + aUid;
    let iter = addressBookManager.directories;
    while (iter.hasMoreElements()) {
        let data = iter.getNext();
        if (data instanceof Ci.nsIAbDirectory) {
            if (data.dirPrefId == dirPrefId) {
                this.mDirectory = data;
                break;
            }
        }
    }
    if (this.mDirectory === null) {
        throw "The addressbook "+aUid+" doesn't exists";
    }
}


ObmAddressBook.prototype = {
    QueryInterface: XPCOMUtils.generateQI([Ci.obmIAddressBook]),

    get uid() {
        return this.mUid;
    },

    get directory () {
        return this.mDirectory;
    },

    getCard : function (aId) {
        var aContact = this.mDirectory.getCardFromProperty("ObmId", aId, false);
        if (!aContact) {
            throw ("Unexpected id (in addressbook " + this.mUid + "): " + aId);
        }
        return aContact;
    },

    modifyCard : function (aCard) {
        this.mDirectory.modifyCard(aCard);
    },

    addCard : function (aCard) {
        this.mDirectory.addCard(aCard);
    },

    deleteCard : function (aCard) {
        var cardsToDelete = Cc["@mozilla.org/array;1"].createInstance(Ci.nsIMutableArray);
        cardsToDelete.appendElement(aCard, false);
        this.directory.deleteCards(cardsToDelete);
    },
    cardForEmailAddress : function (aEmail) {
        return this.mDirectory.cardForEmailAddress(aEmail);
    },
    containsCard : function (aCard) {
        try {
            if (this.mDirectory.cardForEmailAddress(aCard.primaryEmail)) {
                return true;
            }
        }
        catch(e) { // we may have an NS_ERROR_NOT_AVAILABLE on asynchronous ab
        }
        return false;
    }
}

/**
 * main object to access to OBM address books
 */
function ObmAddressBooksManager () {
    try {
        Components.utils.import("resource://obm-extension/modules/UtilsOBM.jsm");
    } catch (e) {
        Components.utils.reportError(e);
    }

    // Init logger
    this.logger = Cc["@obm.org/calendar/logger;1"].createInstance(Ci.obmILogger);
    var writer = Cc["@obm.org/calendar/writer;1?type=unixfile"].getService(Ci.obmILoggerWriter);
    this.logger.setLevelWriter(this.logger.LOAD_LEVEL, writer);
    this.logger.setHeader("ObmAddressBooksManager.js: ");
    this.logger.debug("new ObmAddressBooksManager()", false, false);
};

ObmAddressBooksManager.prototype = {
    QueryInterface: XPCOMUtils.generateQI([Ci.obmIAddressBooksManager]),
    classID: Components.ID("{f6cdfa9e-6961-4d9e-a7cc-612783c1bb0b}"),
    contractID: "@obm.org/addressbooksmanager;1",
    classDescription: "OBM AddressBooks Manager",

    get addressBookManager() {
        let abManager = Components.classes["@mozilla.org/abmanager;1"]
                                  .getService(Components.interfaces.nsIAbManager);
        this.__defineGetter__("addressBookManager", function() { return abManager });
        return abManager;
    },

    _addAddressBookNames: function(newName) {
        let namesPref = UtilsOBM.getPrefSafe("extensions.obm.addressbooks");
        var names = namesPref ? namesPref.split(",") : [];
        var value = "";
        if (names.length == 0) {
            value = newName;
        } else {
            for(var i=0; i<names.length; i++) {
                if (names[i] == newName) {
                    return;
                }
            }
            value = namesPref + "," + newName;
        }
        UtilsOBM.setPrefSafe("extensions.obm.addressbooks", value);
    },

    _removeAddressBookNames: function(name) {
        let namesPref = UtilsOBM.getPrefSafe("extensions.obm.addressbooks");
        var names = namesPref ? namesPref.split(",") : [];
        var newNames = [];
        for (var i=0; i<names.length; i++) {
            if (names[i] != name) {
                newNames.push(names[i]);
            }
        }
        UtilsOBM.setPrefSafe("extensions.obm.addressbooks", newNames.join(","));
    },

    getAddressBookUids: function(count) {
        let names = UtilsOBM.getPrefSafe("extensions.obm.addressbooks");
        let list = names ? names.split(",") : [];
        if (count)
            count.value = list.length;
        return list;
    },

    getObmAddressBooks : function (count) {
        let names = UtilsOBM.getPrefSafe("extensions.obm.addressbooks");
        let list = names ? names.split(",") : [];
        if (count) count.value = list.length;
        return list.map(this.getAddressBook, this);
    },

    getAddressBook : function (aUid) {
        return (new ObmAddressBook(aUid)).QueryInterface(Ci.obmIAddressBook);
    },

    getOBMContactsAddressBook : function () {
        this.logger.debug("getOBMContactsAddressBook: ",false,false);
        let list = this.getObmAddressBooks({});

        let expectedName = UtilsOBM.getLocalizedPref("extensions.obm.addressbooks.contacts", "OBM - Contacts");

        // the only way to find the obm contact address book, is to check
        // its name. However, the user could change the name, so
        // we check also if the name does not contain the word "contact"
        let found = null;
        let possible = null;
        let first = null;

        for each (let ad in list) {
            let name = ad.directory.dirName;
            if (name == expectedName) { // original name
                found = ad;
                break;
            }

            if (name.search(/contact/i) != -1 && name.search(/collect/i) == -1) {
                // here we want to add the contact to the addressbook of the current user.
                // unfortunately there is a way to share addressbooks.
                // so a user having another user addressbook will have somethin like :
                // Mes Contacts
                // Mes contacts (other user)
                // So in the "possible" list we keep the one having the less characters
                // CF Jira OBMFULL-2328
                if ( possible == null || possible.directory.dirName.length > name.length ) {
                    possible = ad;
                }
            }
        }

        if (!found) {
            if (possible) {
                found = possible;
            } else if (list.length) {
                // ok, we don't have the choice, let's take the first OBM ad
                found = list[0];
            }
        }
        return found;
    },

    removeAddressBook : function (aUid) {
        let addr = new ObmAddressBook(aUid);
        if (addr.directory) {
            this.addressBookManager.deleteAddressBook(addr.directory.URI);
        }
        this._removeAddressBookNames(aUid);
    },

    addAddressBook: function (aUid, dirName, abURI, dirPrefId) {
        this.addressBookManager.newAddressBook(dirName, abURI, 2, dirPrefId);
        this._addAddressBookNames(aUid);
    },

    ensureAddressBookExists : function (aUid, aName, aOwnerDisplayName) {
        this.logger.debug("ensureAddressBookExists()", false, false);

        /* Get configured name for "contacts" and "collected_contacts" */
        let dirName = UtilsOBM.getLocalizedPref("extensions.obm.addressbooks." + aName, aName);

        if (aOwnerDisplayName != null && aOwnerDisplayName != "") {
            dirName += " (" + aOwnerDisplayName + ")";
        }

        try {
            let addrb = new ObmAddressBook(aUid);
            if (addrb.directory.dirName != dirName)
                addrb.directory.dirName = dirName;
        } catch(e) {
            let abURI = "moz-abmdbdirectory://" + aUid + ".mab";
            let dirPrefId = "ldap_2.servers." + aUid;

            this.addAddressBook(aUid, dirName, abURI, dirPrefId);
        }
    },

    escapedVCardToAbCard : function (aStringVcard) {
        return this.addressBookManager.escapedVCardToAbCard(aStringVcard);
    }
};

var components = [ObmAddressBooksManager];
if ("generateNSGetFactory" in XPCOMUtils) {
    var NSGetFactory = XPCOMUtils.generateNSGetFactory(components);
} else {
    var NSGetModule = XPCOMUtils.generateNSGetModule(components);
}
