/******************************************************************************
Copyright (C) 2011-2012 Linagora

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License as published by the Free
Software Foundation, either version 3 of the License, or (at your option) any
later version, provided you comply with the Additional Terms applicable for OBM
connector by Linagora pursuant to Section 7 of the GNU Affero General Public
License, subsections (b), (c), and (e), pursuant to which you must notably (i)
retain the “Message sent thanks to OBM, Free Communication by Linagora”
signature notice appended to any and all outbound messages (notably e-mail and
meeting requests), (ii) retain all hypertext links between OBM and obm.org, as
well as between Linagora and linagora.com, and (iii) refrain from infringing
Linagora intellectual property rights over its trademarks and commercial
brands. Other Additional Terms apply, see <http://www.linagora.com/licenses/>
for more details.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU Affero General Public License for more details.

You should have received a copy of the GNU Affero General Public License and
its applicable Additional Terms for OBM along with this program. If not, see
<http://www.gnu.org/licenses/> for the GNU Affero General   Public License
version 3 and <http://www.linagora.com/licenses/> for the Additional Terms
applicable to OBM connectors.
******************************************************************************/
var EXPORTED_SYMBOLS = ["EventComparator", "EventField"];

/**
 * Compare two versions of the same event, returning a list of fields names
 * which are different between the two versions.
 */

var EventField = {
    TITLE: "TITLE",
    PRIORITY: "PRIORITY",
    PRIVACY: "PRIVACY",
    STATUS: "STATUS",
    TRANSPARENCY: "TRANSPARENCY",
    START_DATE: "START_DATE",
    END_DATE: "END_DATE",
    LOCATION: "LOCATION",
    DESCRIPTION: "DESCRIPTION",
    ORGANIZER: "ORGANIZER",
    ATTENDEES: "ATTENDEES",
    RECURRENCE: "RECURRENCE",
    ALARMS: "ALARMS",
    CATEGORIES: "CATEGORIES",

    EX_TITLE: "EX_TITLE",
    EX_PRIORITY: "EX_PRIORITY",
    EX_PRIVACY: "EX_PRIVACY",
    EX_STATUS: "EX_STATUS",
    EX_TRANSPARENCY: "EX_TRANSPARENCY",
    EX_START_DATE: "EX_START_DATE",
    EX_END_DATE: "EX_END_DATE",
    EX_LOCATION: "EX_LOCATION",
    EX_DESCRIPTION: "EX_DESCRIPTION",
    EX_ORGANIZER: "EX_ORGANIZER",
    EX_ATTENDEES: "EX_ATTENDEES",
    EX_ALARMS: "EX_ALARMS",
    EX_CATEGORIES: "EX_CATEGORIES",

    fieldNameToEventField: function(fieldName) {
        switch(fieldName) {
            case "title": return EventField.TITLE;
            case "priority": return EventField.PRIORITY;
            case "privacy": return EventField.PRIVACY;
            case "status": return EventField.STATUS;
            case "startDate": return EventField.START_DATE;
            case "endDate": return EventField.END_DATE;
            case "transparency": return EventField.TRANSPARENCY;
            default: throw new Error("Unknown event field name: " + fieldName);
        }
    }
};

function EventFieldDiff(eventFields) {
    this.eventFields = eventFields;
}

EventFieldDiff.prototype.hasOnlyAlarmChanges = function() {
    return this.eventFields.filter(function(field) {
        return field != EventField.ALARMS && field != EventField.EX_ALARMS;
    }).length == 0;
};

EventFieldDiff.prototype.hasNoChanges = function() {
    return this.eventFields.length == 0;
};

function EventComparator(oldItem, newItem) {
    this.oldItem = oldItem;
    this.newItem = newItem;
    this.initLogger();
}

EventComparator.prototype.initLogger = function() {
    let logger = Components.classes["@obm.org/calendar/logger;1"].createInstance(Components.interfaces.obmILogger);
    let writer = Components.classes["@obm.org/calendar/writer;1?type=unixfile"].getService(Components.interfaces.obmILoggerWriter);
    logger.setLevelWriter(logger.LOAD_LEVEL, writer);
    logger.setHeader("EventComparator: ");
    logger.debug("new EventComparator()", false, false);
    this.logger = logger;
}

EventComparator.prototype.compare = function() {
    let diffs;
    let eventDiffs = []
        .concat(this.compareSimpleFields())
        .concat(this.compareProperties())
        .concat(this.compareCategories())
        .concat(this.compareDateFields())
        .concat(this.compareAttendeeFields())
        .concat(this.compareRecurrences())
        .concat(this.compareAlarms());
    if (this.oldItem.recurrenceId != null && this.newItem.recurrenceId != null) {
        diffs = this.eventDiffsToExDiffs(eventDiffs);
    }
    else {
        diffs = eventDiffs;
    }
    return new EventFieldDiff(diffs);
};

EventComparator.prototype.eventDiffsToExDiffs = function(eventDiffs) {
    var self = this;
    return eventDiffs.map(function(diff) {
        return self.convertEventDiffToExtDiff(diff);
    });
}

EventComparator.prototype.convertEventDiffToExtDiff = function(eventDiff) {
        switch(eventDiff) {
            case EventField.TITLE:
                return EventField.EX_TITLE;
            case EventField.PRIORITY:
                return EventField.EX_PRIORITY;
            case EventField.PRIVACY:
                return EventField.EX_PRIVACY;
            case EventField.STATUS:
                return EventField.EX_STATUS;
            case EventField.TRANSPARENCY:
                return EventField.EX_TRANSPARENCY;
            case EventField.RECURRENCE_START_DATE:
                return EventField.EX_RECURRENCE_START_DATE;
            case EventField.START_DATE:
                return EventField.EX_START_DATE;
            case EventField.END_DATE:
                return EventField.EX_END_DATE;
            case EventField.LOCATION:
                return EventField.EX_LOCATION;
            case EventField.DESCRIPTION:
                return EventField.EX_DESCRIPTION;
            case EventField.ORGANIZER:
                return EventField.EX_ORGANIZER;
            case EventField.ATTENDEES:
                return EventField.EX_ATTENDEES;
            case EventField.ALARMS:
                return EventField.EX_ALARMS;
            case EventField.CATEGORIES:
                return EventField.EX_CATEGORIES;
            default:
                throw new Error("Unknown field type: " + eventDiff);
        }
}

EventComparator.prototype.compareSimpleFields = function() {
    let fieldsWithDiff = [];
    for each (let fieldName in ['title', 'priority', 'privacy', 'status']) {
        let oldValue = this.oldItem[fieldName];
        let newValue = this.newItem[fieldName];
        if (oldValue != newValue) {
            fieldsWithDiff.push(EventField.fieldNameToEventField(fieldName));
        }
    }
    return fieldsWithDiff;
};

EventComparator.prototype.compareProperties = function() {
    let fieldsWithDiff = [];
    if (!this.arePropertiesEqual('transp')) {
        fieldsWithDiff.push(EventField.TRANSPARENCY);
    }
    if (!this.arePropertiesEqual('location')) {
        fieldsWithDiff.push(EventField.LOCATION);
    }
    if (!this.arePropertiesEqual('description')) {
        fieldsWithDiff.push(EventField.DESCRIPTION);
    }
    return fieldsWithDiff;
};

EventComparator.prototype.compareDateFields = function() {
    let fieldsWithDiff = [];
    for each (let fieldName in ['startDate', 'endDate']) {
        let oldValue = this.oldItem[fieldName];
        let newValue = this.newItem[fieldName];
        if (!this.areDatesEqual(oldValue, newValue)) {
            fieldsWithDiff.push(EventField.fieldNameToEventField(fieldName));
        }
    }
    return fieldsWithDiff;
};

EventComparator.prototype.compareCategories = function() {
    let oldCategories = this.oldItem.getCategories({});
    let newCategories = this.newItem.getCategories({});
    let oldCategoriesString = oldCategories.map(function(item){item.replace(/#/, '\\#')}).sort().join("#");
    let newCategoriesString = newCategories.map(function(item){item.replace(/#/, '\\#')}).sort().join("#");
    return oldCategoriesString != newCategoriesString ?
        [EventField.CATEGORIES] :
        [];
}

EventComparator.prototype.compareAttendeeFields = function() {
    let organizerDiff = this.compareOrganizer();
    let attendeesDiff = this.compareAttendees();
    return organizerDiff.concat(attendeesDiff);
};

EventComparator.prototype.compareOrganizer = function() {
    let oldValue = this.oldItem.organizer;
    let newValue = this.newItem.organizer;
    return this.areAttendeesEqual(oldValue, newValue) ? [] : [EventField.ORGANIZER];
};

EventComparator.prototype.compareAttendees = function() {
    let oldValue = this.oldItem.getAttendees({});
    let newValue = this.newItem.getAttendees({});
    return this.areAttendeesTheSame(oldValue, newValue) ? [] : [EventField.ATTENDEES];
};

EventComparator.prototype.compareRecurrences = function() {
    // only compare the difference in negative exceptions
    let oldRecurrenceExceptions = {};
    let oldRecurrenceExceptionsCount = 0;
    if (this.oldItem.recurrenceInfo != null && this.newItem.recurrenceInfo == null ||
        this.oldItem.recurrenceInfo == null && this.newItem.recurrenceInfo != null) {
        return [EventField.RECURRENCE];
    }
    if (this.oldItem.recurrenceInfo == null && this.newItem.recurrenceInfo == null) {
        return [];
    }

    let oldRecurrenceItemStrings = [];
    let oldRecurrenceItems = this.oldItem.recurrenceInfo.getRecurrenceItems({});
    for each (let oldRecurrenceItem in oldRecurrenceItems) {
        oldRecurrenceItemStrings.push(oldRecurrenceItem.icalProperty.icalString);
    }

    let newRecurrenceItemStrings = [];
    let newRecurrenceItems = this.newItem.recurrenceInfo.getRecurrenceItems({});
    for each (let newRecurrenceItem in newRecurrenceItems) {
        newRecurrenceItemStrings.push(newRecurrenceItem.icalProperty.icalString);
    }
    let oldRecurrenceItemString = oldRecurrenceItemStrings.join(";;;");
    let newRecurrenceItemString = newRecurrenceItemStrings.join(";;;");
    return oldRecurrenceItemString == newRecurrenceItemString ?
        [] : [EventField.RECURRENCE];
};

EventComparator.prototype.compareAlarms = function() {
    let oldAlarmHash = {};
    let oldAlarms = this.oldItem.getAlarms({});
    for each (let oldAlarm in oldAlarms) {
        oldAlarmHash[oldAlarm.icalString] = true;
    }
    let alarmInBothOldAndNewItemsCount = 0;
    for each (let newAlarm in this.newItem.getAlarms({})) {
        if (oldAlarmHash[newAlarm.icalString]) {
            alarmInBothOldAndNewItemsCount++;
        } else {
            return [EventField.ALARMS];
        }
    }
    return oldAlarms.length == alarmInBothOldAndNewItemsCount ? [] : [EventField.ALARMS];
};
EventComparator.prototype.areDatesEqual = function(oldValue, newValue) {
    if (oldValue == null && newValue != null || oldValue != null && newValue == null) {
        return false;
    }
    if (oldValue == null && newValue == null) {
        return true;
    }
    return oldValue.compare(newValue) == 0;
};

EventComparator.prototype.areDurationsEqual = function(oldValue, newValue) {
    return oldValue.compare(newValue) == 0;
};

EventComparator.prototype.areAttendeesEqual = function(oldValue, newValue) {
    return oldValue.id == newValue.id
        && oldValue.role == newValue.role
        && oldValue.participationStatus == newValue.participationStatus
        && oldValue.userType == newValue.userType;
};

EventComparator.prototype.arePropertiesEqual = function(propName) {
    let fieldsWithDiff = [];
    let oldPropValue = this.oldItem.getProperty(propName);
    let newPropValue = this.newItem.getProperty(propName);
    return oldPropValue == newPropValue;
}


EventComparator.prototype.areAlarmsEqual = function(oldValue, newValue) {
    return oldValue.toString() == newValue.toString();
};

EventComparator.prototype.areAttendeesTheSame = function(oldValue, newValue) {
    if (oldValue.length != newValue.length) {
        return false;
    }
    let oldAttendeeIdToAttendee = {};
    for each (let oldAttendee in oldValue) {
        oldAttendeeIdToAttendee[oldAttendee.id] = oldAttendee;
    }
    let attendeesInBothOldAndNewValueCount = 0;
    for each (let newAttendee in newValue) {
        let oldAttendee = oldAttendeeIdToAttendee[newAttendee.id];
        if (oldAttendee == null || !this.areAttendeesEqual(oldAttendee, newAttendee)) {
            return false;
        }
        attendeesInBothOldAndNewValueCount++;
    }
    return attendeesInBothOldAndNewValueCount == newValue.length;
};
